@@ -17,30 +17,38 @@
 
 void regRF_time_series(double *x, double *seglength, int *tardiff, int *segdiff, 
 		int *xdim, int *sampsize, int *nthsize, int *nrnodes, int *nTree, int *mtry, 
-		int *cat, int *jprint, int *target, int *targetType, int *treeSize, 
+		int *cat, int *jprint, int *oobpred, int *target, int *targetType, int *treeSize, 
 		int *nodedepth, int *nodestatus, int *splitType, int *lDaughter, int *rDaughter, 
-		double *avnode, int *mbest, double *upper, int *keepf, int *replace, int *inbag) {
-
-    double  *xb, xrand;
-    int *in, k, m, n, j, nsample, idx, mdim, keepF, keepInbag, maxdepth;
+		double *avnode, int *mbest, double *upper, int *keepf, int *replace, double *oobpredictions,
+		double *ooberrors, int *inbag) {
 
+    double  *xb, xrand, temp;
+    int *in, *targetcount, k, m, n, j, nsample, idx, mdim;
+    int segmentlength, keepF, keepInbag, maxdepth, oobcount=0;
+	
     nsample = xdim[0];    		//number of series
     mdim = xdim[1];	      		//length series
     keepF = keepf[0];     		//keep forest
     keepInbag = keepf[1]; 		//keep inbag data
     maxdepth=log2(*nrnodes);	//maximum depth
+
     
     GetRNGstate();
 
+	if (*replace) { /* sample */		
+		xb = (double *) Calloc(mdim*(*sampsize), double); //inbag x info
+		in = (int *) Calloc(nsample, int);
+		targetcount = (int *) Calloc(mdim*nsample, int);
+		zeroInt(targetcount, mdim*nsample);
+	}
     /*************************************
      * Start the loop over trees.
      *************************************/
+   
     for (j = 0; j < *nTree; ++j) {     
 		idx = keepF ? j * *nrnodes : 0;		
 		/* Draw a random sample for growing a tree. */
 		if (*replace) { /* sample */		
-			xb = (double *) S_alloc(mdim * *sampsize, sizeof(double));  //inbag x info
-			in = (int *) S_alloc(nsample, sizeof(int));
 			zeroInt(in, nsample);
 			for (n = 0; n < *sampsize; ++n) {
 				xrand = unif_rand();
@@ -57,9 +65,29 @@ void regRF_time_series(double *x, double *seglength, int *tardiff, int *segdiff,
 			regTree_time_series(xb, seglength + j, *tardiff, *segdiff, maxdepth, mdim, *sampsize, lDaughter + idx, rDaughter + idx,
                 upper + idx, avnode + idx, nodedepth + idx, nodestatus + idx, splitType + idx, *nrnodes,
                 treeSize + j, *nthsize, *mtry, mbest + idx, target + j, targetType + j, cat);
-           
-            Free(xb);
-            Free(in);
+			
+			if (*oobpred) {
+			//	Rprintf("Yalan\n");
+				segmentlength=(int) (mdim*seglength[j]);
+				oobcount=0;            
+				for (n = 0; n < nsample; ++n) {
+					if(in[n]==0){
+						predict_time_series(x + n*mdim, segmentlength, 1, mdim, lDaughter + idx, rDaughter + idx,
+							nodedepth + idx, nodestatus + idx, upper + idx, mbest + idx, splitType + idx,
+							avnode + idx, maxdepth, target[j], oobpredictions + n*mdim, targetcount + n*mdim); 			
+						//error adama sart			
+					}
+					for (m = 0; m < mdim; ++m) {
+						if(targetcount[m+n*mdim]>0){
+							temp=oobpredictions[m+n*mdim]/targetcount[m+n*mdim];
+							ooberrors[j]=ooberrors[j]+pow(x[m+n*mdim]-temp,2);
+							oobcount++;
+						}
+					}	
+				}  
+				ooberrors[j]=ooberrors[j]/oobcount;   
+			}
+					       
 		}  else { //use all data
 			regTree_time_series(x, seglength + j, *tardiff, *segdiff, maxdepth, mdim, nsample, lDaughter + idx, rDaughter + idx,
                 upper + idx, avnode + idx, nodedepth + idx, nodestatus + idx, splitType + idx, *nrnodes,
@@ -71,7 +99,23 @@ void regRF_time_series(double *x, double *seglength, int *tardiff, int *segdiff,
     }
     PutRNGstate();
     /* end of tree iterations=======================================*/
-
+    
+	if (*replace) { /* free memory */
+	   if (*oobpred) {
+		   for (n = 0; n < nsample; ++n) {
+			   for (m = 0; m < mdim; ++m) {
+					if(targetcount[m+n*mdim]>0){
+						oobpredictions[m+n*mdim]=oobpredictions[m+n*mdim]/targetcount[m+n*mdim];
+					} else {
+						oobpredictions[m+n*mdim]=-999;
+					}  
+			   }		
+		   }
+	   }
+       Free(xb);
+       Free(in);
+       Free(targetcount);
+	}
 }
 		
 void regForest_similarity(double *x, double *y, int *n, int *ny,
@@ -112,7 +156,6 @@ void regForest_similarity(double *x, double *y, int *n, int *ny,
 		predictRepresentation_time_series(y, segmentlength, *ny, *mdim, lDaughter + idx1, rDaughter + idx1,
 				nodedepth + idx1, nodestatus + idx1, xsplit + idx1, mbest + idx1, splitType + idx1, nodetest, *maxdepth); 
 
-
 		for (k = 0; k < *nrnodes; k++) {
 			if(tempnodestatus[k]==NODE_TERMINAL){ 
 				for(j = 0; j < (*ny); j++){	
@@ -200,10 +243,133 @@ void regForest_represent(double *x, int *n, int *whichtree,
 	Free(tempnodestatus);				
 }
 
+void regForest_predict(double *x, int *n, int *whichtree,
+			double *seglength, int *mdim, int *ntree, 
+			int *lDaughter, int *rDaughter, int *nodestatus, int *nodedepth,
+			int *nrnodes, double *xsplit, int *mbest, int *splitType, 
+			double *nodepred, int *treeSize, int *target, int *maxdepth, 
+			double *prediction, int *targetcount) {
+				
+	int i, j, idx1, startind, endind;
+    int segmentlength;
+
+	if(*whichtree>0){ //single tree
+		startind=*whichtree-1;
+		endind=startind+1;
+		idx1=*nrnodes*(*whichtree-1);
+	}
+	else {	//ensemble
+		startind=0;
+		endind=*ntree;
+		idx1 = 0;	
+	}
+	
+	zeroDouble(prediction,(*n)*(*mdim));
+	zeroInt(targetcount,(*mdim));
+	
+	for (i = startind; i < endind; i++) {
+		segmentlength=(int) (*mdim*seglength[i]);
+
+		predict_time_series(x, segmentlength, *n, *mdim, lDaughter + idx1, rDaughter + idx1,
+				nodedepth + idx1, nodestatus + idx1, xsplit + idx1, mbest + idx1, splitType + idx1,
+				nodepred + idx1, *maxdepth, target[i], prediction, targetcount);
+			
+	   idx1 += *nrnodes; 
+	}	
+
+	for (i = 0; i < *n; i++) {
+		for (j = 0; j < *mdim; j++) {
+			if(targetcount[j]>0){
+				prediction[j+i*(*mdim)]=prediction[j+i*(*mdim)]/targetcount[j];
+			} else {
+				prediction[j+i*(*mdim)]=NA_indicator;
+			}
+		}
+	}
+		
+}
+
+
+void regForest_pattern(double *x, int *n, int *whichtree,
+			int *whichterminal, double *seglength, int *mdim, int *ntree, 
+			int *lDaughter, int *rDaughter, int *nodestatus, int *nodedepth,
+			int *nrnodes, double *xsplit, int *mbest, int *splitType, 
+			int *treeSize, int *maxdepth, int *target, int *targetType, 
+			double *predictpattern, double *targetpattern) {
+				
+	int i, j, k, m, idx1, startind, endind, nodecount, termnodecount;
+    int segmentlength, totx, termid, curtree, lastk;
+
+	curtree=*whichtree-1;
+	idx1=*nrnodes*curtree;
+
+	for (i = 0; i < *n; i++) {
+		for (j = 0; j < *mdim; j++) {
+			predictpattern[j+i*(*mdim)]=NA_indicator;
+			targetpattern[j+i*(*mdim)]=NA_indicator;
+		}
+	}
+
+	termnodecount = 0;
+	for (k = 0; k < *nrnodes; k++) {
+		if(nodedepth[idx1+k]==*maxdepth || nodestatus[idx1+k]==NODE_TERMINAL)
+			termnodecount++;
+				
+		if(termnodecount==*whichterminal)
+			break;
+	}	
+	termid=k;
+	segmentlength=(int) (*mdim*seglength[curtree]);
+	for (j = 0; j < segmentlength ; j++) {
+		for (i = 0; i < *n; i++) {
+			k = 0;
+			while (nodestatus[idx1+k] != NODE_TERMINAL && nodedepth[idx1+k] < *maxdepth) {
+				m = mbest[idx1+k] - 1;
+				lastk = k;
+				if(splitType[idx1+k]==OBS_SERIES){
+					if(m+j>(*mdim)-1){
+						k = (x[m+j-(*mdim)+i*(*mdim)] <= xsplit[idx1+k]) ?
+						lDaughter[idx1+k] - 1 : rDaughter[idx1+k] - 1;
+					} else {
+						k = (x[m+j+i*(*mdim)] <= xsplit[idx1+k]) ?
+						lDaughter[idx1+k] - 1 : rDaughter[idx1+k] - 1;						
+					}
+				}  else if(splitType[idx1+k]==DIFF_SERIES){
+					if(m+j>(*mdim)-2){
+						k = ((x[m+(j+2)-(*mdim)+i*(*mdim)]-x[m+j-(*mdim)+1+i*(*mdim)]) <= xsplit[idx1+k]) ?
+						lDaughter[idx1+k] - 1 : rDaughter[idx1+k] - 1;
+					} else {
+						k = ((x[m+(j+1)+i*(*mdim)]-x[m+j+i*(*mdim)]) <= xsplit[idx1+k]) ?
+						lDaughter[idx1+k] - 1 : rDaughter[idx1+k] - 1;					
+					}				
+				}		
+			} 
+			
+			if(termid==k){
+				m = target[curtree] - 1;
+				if(m+j>(*mdim)-1){
+					targetpattern[m+j-(*mdim)+i*(*mdim)] = x[m+j-(*mdim)+i*(*mdim)];
+				} else {
+					targetpattern[m+j+i*(*mdim)] = x[m+j+i*(*mdim)];			
+				}
+
+				m = mbest[idx1] - 1;
+				if(m+j>(*mdim)-1){
+					predictpattern[m+j-(*mdim)+i*(*mdim)] = x[m+j-(*mdim)+i*(*mdim)];
+				} else {
+					predictpattern[m+j+i*(*mdim)] = x[m+j+i*(*mdim)];			
+				}
+		 			
+			}
+		}
+	}
+		
+}
+
 /*************************************
  * Codes to use for future development
 *************************************/
-     
+
 /*
 void regForest_represent(double *x, double *seglength, int *mdim, int *n,
                int *ntree, int *lDaughter, int *rDaughter,
@@ -260,7 +426,68 @@ void regForest_represent(double *x, double *seglength, int *mdim, int *n,
 	Free(tempnodestatus);
 }
 
+void regForest_findNN(double *x, double *y, int *n, int *ny,
+			double *seglength, int *mdim, int *ntree, 
+			int *lDaughter, int *rDaughter, int *nodestatus, int *nodedepth,
+			int *nrnodes, double *xsplit, int *mbest, int *splitType, 
+			int *treeSize, int *maxdepth, int *K, int *result) {
+				   		                    
+    int i, j, k, m, idx1, segmentlength;
+    int *noderef, *nodetest, *tempnodestatus, dist, bestsofar;
+	
 
+    noderef = (int *) Calloc((*nrnodes), int);	
+    nodetest = (int *) Calloc((*nrnodes), int);
+    tempnodestatus = (int *) Calloc((*ntree)*(*nrnodes), int);
+
+	zeroInt(tempnodestatus, (*ntree)*(*nrnodes));
+		
+	// based on the maxdepth setting identify terminal nodes
+	for (i = 0; i < *ntree; i++) {
+		idx1 = 0;
+		for (k = 0; k < *nrnodes; k++) {
+			if(nodedepth[idx1+k]==*maxdepth || nodestatus[idx1+k]==NODE_TERMINAL){
+				tempnodestatus[idx1+k]=NODE_TERMINAL;
+			}
+		}
+		idx1 += *nrnodes; 
+	}
+		
+	for(j = 0; j < (*ny); j++){	
+		bestsofar=30000;
+		for(m = 0; m < (*n); m++){
+			dist=0;
+			for (i = 0; i < *ntree; i++) {
+				idx1 = 0;
+				segmentlength=(int) (*mdim*seglength[i]);			
+				zeroInt(noderef, (*nrnodes));
+				zeroInt(nodetest, (*nrnodes));
+				predictRepresentation_time_series(x + m*(*mdim), segmentlength, 1, *mdim, lDaughter + idx1, rDaughter + idx1,
+						nodedepth + idx1, nodestatus + idx1, xsplit + idx1, mbest + idx1, splitType + idx1, noderef, *maxdepth);
+				predictRepresentation_time_series(y+ j*(*mdim), segmentlength, 1, *mdim, lDaughter + idx1, rDaughter + idx1,
+						nodedepth + idx1, nodestatus + idx1, xsplit + idx1, mbest + idx1, splitType + idx1, nodetest, *maxdepth); 		
+				for (k = 0; k < *nrnodes; k++) {
+					if(tempnodestatus[idx1+k]==NODE_TERMINAL){ 
+						dist=dist+abs(noderef[k]-nodetest[k]);
+					}
+				}
+				idx1 += *nrnodes; 	
+				if(bestsofar<dist)
+					break;
+			}
+
+			if(dist<bestsofar){
+				bestsofar=dist;
+				result[j]=m+1;
+			}
+		}
+	}	
+	
+	Free(noderef);
+	Free(nodetest);
+	Free(tempnodestatus);
+}
+* 
 void regForest_time_series(double *x, double *seglength, int *mdim, int *n,
                int *ntree, int *lDaughter, int *rDaughter,
                int *nodestatus, int *nrnodes, double *xsplit, int *mbest, int *treeSize, int *max_depth, int *nodes, int *nodex) {
